from admin_tools.dashboard import modules, Dashboard, AppIndexDashboard
from admin_tools.utils import get_admin_site_name
from django.apps import apps
from django.urls import reverse
from django.utils.module_loading import import_string
from django.utils.translation import ugettext_lazy as _


def get_children_dashboard():
    children = []
    for app in apps.get_app_configs():
        try:
            dashboard = import_string(f'{app.name}.dashboard.MainDashboard')
            children.append(dashboard())
        except BaseException:
            pass
    return children


def get_children_app_dashboard(app_title):
    children = []
    try:
        app_label = app_title.lower()
        app_name = apps.get_app_config(app_label).name
        dashboard = import_string(f'{app_name}.dashboard.DefaultDashboard')
        children.append(dashboard())
    except BaseException:
        pass
    return children


class CustomIndexDashboard(Dashboard):
    columns = 3

    def init_with_context(self, context):
        site_name = get_admin_site_name(context)
        # append a link list module for "quick links"
        self.children.append(modules.LinkList(
            _('Quick links'),
            layout='inline',
            draggable=False,
            deletable=False,
            collapsible=False,
            children=[
                [_('Return to site'), '/'],
                [_('Change password'),
                 reverse('%s:password_change' % site_name)],
                [_('Log out'), reverse('%s:logout' % site_name)],
            ]
        ))
        self.children += get_children_dashboard()

        # append an app list module for "Administration"
        self.children.append(modules.AppList(
            _('Administration'),
            models=('django.contrib.*',),
        ))

        # append a recent actions module
        self.children.append(modules.RecentActions(_('Recent Actions'), 5))

        # append a feed module
        self.children.append(modules.Feed(
            _('Latest Django News'),
            feed_url='http://www.djangoproject.com/rss/weblog/',
            limit=5
        ))

        # append another link list module for "support".
        self.children.append(modules.LinkList(
            _('Support'),
            children=[
                {
                    'title': _('Django documentation'),
                    'url': 'http://docs.djangoproject.com/',
                    'external': True,
                },
                {
                    'title': _('Django "django-users" mailing list'),
                    'url': 'http://groups.google.com/group/django-users',
                    'external': True,
                },
                {
                    'title': _('Django irc channel'),
                    'url': 'irc://irc.freenode.net/django',
                    'external': True,
                },
            ]
        ))


class CustomAppIndexDashboard(AppIndexDashboard):
    """
    Custom app index dashboard for office2019.thux.it.
    """

    def init_with_context(self, context):
        """
        Use this method if you need to access the request context.
        """
        self.title = f"{self.app_title} dashboard"

        # add application dashboard
        self.children += get_children_app_dashboard(context['app_label'])

        # append a model list module and a recent actions module
        self.children += [
            # modules.ModelList(self.app_title, self.models),
            modules.RecentActions(
                _('Recent Actions'),
                include_list=self.get_app_content_types(),
                limit=5
            )
        ]
