# -*- coding: utf-8 -*-

from django.contrib import messages
from django.urls import reverse
from django.utils.html import strip_tags
from django.utils.translation import gettext_lazy as _
from django.views.generic.edit import FormView


class ThuxFormView(FormView):
    template_name = 'admin/change_form.html'
    form_class = None
    app_label = None
    model_name = None
    title = None
    app_title = None
    model_title = None
    model = None
    url = 'change_form'

    def get_template_names(self):
        """
        Return a list of template names to be used for the request.
        """
        template_name = f"admin/generic_form.html"
        return [
            template_name
        ]

    def get_breadcrumbs(self):
        """
        Return a list of breadcrumbs to be used.
        """
        return (
            (_('Home'), reverse('admin:index')),
            (self.app_title, reverse('admin:app_list', kwargs={'app_label': '{}'.format(self.app_label)})),
            (self.model_title, reverse('admin:{}_{}_changelist'.format(self.app_label, self.model_name))),
            (self.title,),
        )

    def get_context_data(self, **kwargs):
        """
        Override default context with add breadcrumbs and success url.
        """
        context = super().get_context_data(**kwargs)
        breadcrumbs = self.get_breadcrumbs()
        opts = self.model._meta
        context.update({
            'opts': opts,
            'title': self.title,
            'breadcrumbs': breadcrumbs,
            'success_url': self.get_success_url
        })
        return context

    def get_success_url(self):
        """
        Return "success url" to use.
        """
        app_label = self.app_label
        model_name = self.model_name
        return reverse('admin:{}_{}_changelist'.format(app_label, model_name))

    def form_valid(self, form):
        res, obj_or_msg = form.do()
        if res:
            for obj in obj_or_msg:
                messages.success(self.request, obj)
            return super().form_valid(form)
        else:
            messages.error(self.request, obj_or_msg)
        return self.render_to_response(self.get_context_data(form=form))

    def form_invalid(self, form):
        for error in form.errors:
            error_msg = " {}: {}".format(error, strip_tags(form.errors[error]))
            messages.error(self.request, error_msg)
        return self.render_to_response(self.get_context_data(form=form))
