# coding: utf-8

from admin_tools.menu import items, Menu
from django.apps import apps as d_apps
from django.urls import reverse
from django.utils.translation import ugettext as _
from importlib import import_module


class ThuxMenu(Menu):
    def has_perms(self, perms):
        if not perms:
            return False
        for perm in perms:
            user = self.request.user
            app_label, value = perm.split('.')
            # app_label.*: user has at least one perm for 'app_label'
            if value == '*' and user.has_module_perms(app_label):
                return True
            if user.has_perm(perm):
                return True
        return False

    def get_menu_item(self, item):
        try:
            item = item.copy()
            title = _(item.pop('title')) if 'title' in item else None
            perms = item.pop('perms') if 'perms' in item else None
            if not self.has_perms(perms):
                return
            url = item.pop('url') if 'url' in item else None
            url_name = item.pop('url_name') if 'url_name' in item else None
            if url_name and isinstance(url_name, dict):
                url = reverse(**url_name)
            elif url_name and isinstance(url_name, str):
                url = reverse(url_name)
            children = item.pop('children') if 'children' in item else []
            item_children = []
            if children:
                for child in children:
                    menu_item = self.get_menu_item(child)
                    if menu_item:
                        item_children.append(menu_item)
            menu = items.MenuItem(title, url, children=item_children, **item)
            return menu
        except:
            pass

    def get_menu_items(self):
        children = []
        for app in d_apps.get_app_configs():
            try:
                module = import_module(f'{app.name}.jmenu')
                for item in module.menu_items:
                    menu_item = self.get_menu_item(item)
                    if menu_item:
                        children.append(menu_item)
            except BaseException:
                pass
        return children

    def __init__(self, *args, **kwargs):
        Menu().__init__(*args, **kwargs)
        self.request = None

    def init_with_context(self, context, **kwargs):
        Menu.init_with_context(self, context, **kwargs)
        self.request = context.request
        self.children = []

        # ADMINISTRATION menu
        self.children += [
            items.MenuItem(_('Home'), reverse('admin:index')),
        ]
        self.children += self.get_menu_items()
