# -*- coding: utf-8 -*-

from django.urls import re_path
from django.utils.translation import gettext_lazy as _


class LinkClassMixin:
    link_classes = None

    def get_link_classes(self):
        if not self.link_classes:
            raise AttributeError(_("'link_classes' attribute not set for {}").format(self.__class__.__name__))
        return self.link_classes

    def check_attributes(self):
        for link_class in self.get_link_classes():
            url = getattr(link_class, 'url', None)
            url_name = getattr(link_class, 'url_name', None)
            button = getattr(link_class, 'button', None)
            title = getattr(link_class, 'title', None)
            view_class = getattr(link_class, 'view_class', None)

            if url is None:
                raise ValueError(_("'url' value not found"))
            if url_name is None:
                raise ValueError(_("'url_name' value not found"))
            if button is None:
                raise ValueError(_("'button' value not found"))
            if title is None:
                raise ValueError(_("'title' value not found"))
            if view_class is None:
                raise ValueError(_("'view_class' value not found"))

    def changelist_view(self, request, extra_context=None):
        self.check_attributes()
        extra_context = self.get_object_tools_extra_context(request, extra_context)
        return super().changelist_view(request, extra_context)

    def get_object_tools_extra_context(self, request, extra_context=None):
        extra_context = extra_context or {}
        extra_context['object_tools'] = self.get_object_tools(request)
        return extra_context

    def get_object_tools(self, request):
        my_ot = []
        for link_class in self.get_link_classes():
            if link_class.button is True:
                my_ot.append({'admin_urlname': link_class.url, 'title': link_class.title})
        return my_ot

    def get_urls(self):
        urls = super().get_urls()
        my_urls = []
        for link_class in self.get_link_classes():
            my_urls.append(
                re_path(
                    r'^{}/$'.format(link_class.url),
                    self.admin_site.admin_view(link_class.view_class.as_view()),
                    name='{}_{}_{}'.format(
                        self.model._meta.app_label, self.model._meta.model_name, link_class.url_name
                    )
                )
            )
        return my_urls + urls
